use egui::{Sense, Stroke, Vec2};

use super::{CompGetter, CompUi};


#[derive(Debug, Default)]
pub struct Player {
    slider_progress: usize,
    is_playing: bool,
}

component_register!(Player);


impl CompUi for Player {
    fn draw(ui: &mut egui::Ui, _: &mut crate::GuiState) -> crate::Result<()> {
        let theme = xmpd_settings::Settings::get()?.theme.clone();
        let avail = ui.available_size();
        ui.vertical_centered_justified(|ui| {
            ui.add_space(3.0);
            ui.horizontal(|ui| {
                {
                    ui.add_space(avail.x * 0.05 / 2.0);
                    let mut slf = handle_error_ui!(Player::get());
                    let slider = egui::Slider::new(&mut slf.slider_progress, 0..=100)
                        .show_value(false);
                    ui.style_mut().spacing.slider_width = avail.x * 0.90;
                    let s = Stroke {
                        color: theme.accent_color,
                        width: 2.0
                    };
                    ui.style_mut().visuals.widgets.inactive.fg_stroke = s;
                    ui.style_mut().visuals.widgets.active.fg_stroke = s;
                    ui.style_mut().visuals.widgets.hovered.fg_stroke = s;
                    ui.add(slider);
                    ui.label("00:00");
                }
            });
            ui.horizontal(|ui| {
                ui.add_space((avail.x / 2.0) - 16.0 - 8.0 - ui.spacing().item_spacing.x);
                let pp = if handle_error_ui!(Player::get()).is_playing {
                    crate::data::PAUSE_ICON
                } else {
                    crate::data::PLAY_ICON
                };


                let prev = egui::Image::new(crate::data::PREV_ICON)
                    .tint(theme.accent_color)
                    .sense(Sense::click())
                    .max_size(Vec2::new(16.0, 16.0));
                let pp = egui::Image::new(pp)
                    .tint(theme.accent_color)
                    .sense(Sense::click())
                    .max_size(Vec2::new(16.0, 16.0));
                let next = egui::Image::new(crate::data::NEXT_ICON)
                    .tint(theme.accent_color)
                    .sense(Sense::click())
                    .max_size(Vec2::new(16.0, 16.0));
                if ui.add(prev).clicked() {}
                if ui.add(pp).clicked() {
                    let mut slf = handle_error_ui!(Player::get());
                    slf.is_playing = !slf.is_playing;
                }
                if ui.add(next).clicked() {}
            });
            ui.add_space(3.0);
        });
        Ok(())
    }
}
