use std::str::FromStr;

use url::Url;
use uuid::Uuid;

use crate::{song::SourceType, store, Manifest};

use super::{Query, QueryType};


pub struct QuerySong;
impl QueryType for QuerySong {}

impl Query<QuerySong> {
    pub fn id(&self) -> &Uuid {
        if let Some(id) = &self.song {
            return id;
        }
        unreachable!()
    }
    pub fn name<'a, ST: store::BaseStore + Clone>(&self, manifest: &'a Manifest<ST>) -> Option<&'a str> {
        let pl = manifest.store().get_song(self.id())?;
        Some(pl.name())
    }
    pub fn set_name<ST: store::BaseStore + Clone>(&self, manifest: &mut Manifest<ST>, name: &str) -> Option<()> {
        let pl = manifest.store_mut().get_song_mut(self.id())?;
        pl.set_name(name);
        Some(())
    } 
    pub fn author<'a, ST: store::BaseStore + Clone>(&self, manifest: &'a Manifest<ST>) -> Option<&'a str> {
        let pl = manifest.store().get_song(self.id())?;
        Some(pl.author())
    }
    pub fn set_author<ST: store::BaseStore + Clone>(&self, manifest: &mut Manifest<ST>, author: &str) -> Option<()> {
        let pl = manifest.store_mut().get_song_mut(self.id())?;
        pl.set_author(author);
        Some(())
    }
    pub fn url<'a, ST: store::BaseStore + Clone>(&self, manifest: &'a Manifest<ST>) -> Option<&'a Url> {
        let pl = manifest.store().get_song(self.id())?;
        Some(pl.url())
    }
    pub fn url_as_str<'a, ST: store::BaseStore + Clone>(&self, manifest: &'a Manifest<ST>) -> Option<&'a str> {
        Some(self.url(manifest)?.as_str())
    }
    pub fn set_url<ST: store::BaseStore + Clone>(&self, manifest: &mut Manifest<ST>, url: &Url) -> Option<()> {
        let pl = manifest.store_mut().get_song_mut(self.id())?;
        pl.set_url(url);
        Some(())
    }
    pub fn set_url_from_str<ST: store::BaseStore + Clone>(&self, manifest: &mut Manifest<ST>, url: &str) -> Option<()> {
        let Ok(url) = Url::from_str(url) else {return None};
        self.set_url(manifest, &url);
        Some(())
    }
    pub fn source_type<'a, ST: store::BaseStore + Clone>(&self, manifest: &'a Manifest<ST>) -> Option<&'a SourceType> {
        let pl = manifest.store().get_song(self.id())?;
        Some(pl.source_type())
    }
    pub fn set_source_type<ST: store::BaseStore + Clone>(&self, manifest: &mut Manifest<ST>, source_type: &SourceType) -> Option<()> {
        let pl = manifest.store_mut().get_song_mut(self.id())?;
        pl.set_source_type(source_type);
        Some(())
    } 

}


